/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsIAutoCompleteResult.idl"

interface nsIMdbTable;
interface nsIMdbEnv;
interface nsIMdbRow;

%{ C++
#include "mdb.h"
%}

native mdb_scope(mdb_scope);
native mdb_column(mdb_column);

/* noscript */
[uuid(e6396544-921d-4776-aa62-8bf2dc1ae058)]
interface nsIAutoCompleteBaseResult : nsIAutoCompleteResult
{
  void setSearchString(in AString searchString);
  void setErrorDescription(in AString errorDescription);
  void setDefaultIndex(in long defaultIndex);
  void setSearchResult(in unsigned long searchResult);
};

/* noscript */
[uuid(f7e7b341-2d5c-4d04-9f51-496cb2ce178c)]
interface nsIAutoCompleteMdbResult : nsIAutoCompleteBaseResult
{
  const short kUnicharType = 0;
  const short kCharType    = 1;
  const short kIntType     = 2;

  void init(in nsIMdbEnv env, in nsIMdbTable table);

  void setTokens(in mdb_scope valueToken, in short valueType,
		 in mdb_scope commentToken, in short commentType);

  void addRow(in nsIMdbRow row);

  nsIMdbRow getRowAt(in unsigned long rowIndex);

  AString getRowValue(in nsIMdbRow row, in mdb_column col);

  AUTF8String getUTF8RowValue(in nsIMdbRow row, in mdb_column col);

  long getIntRowValue(in nsIMdbRow row, in mdb_column col);
};
