/*
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Help CSS
#
# The Initial Developer of the Original Code is
# R.J. Keller
# Portions created by the Initial Developer are Copyright (C) 2003
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
*/
@import url("chrome://global/skin/");
@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

.toolbarbutton-1 {
	-moz-box-orient: vertical;
	min-width: 0px;
	list-style-image: url("chrome://help/skin/Toolbar.png");
}

.browserButton {
  -moz-box-orient: vertical;
  min-width: 0px;
  list-style-image: url("chrome://browser/skin/Toolbar.png");
}

.toolbarbutton-1, .browserButton, .toolbarbutton-menubutton-button {
  padding: 5px;
}

.toolbarbutton-1 .toolbarbutton-icon,
.browserButton .toolbarbutton-icon {
  -moz-margin-end: 0px;
}

#helpToolbar[mode="full"] .toolbarbutton-1,
#helpToolbar[mode="full"] .browserButton,
#helpToolbar[mode="full"] .toolbarbutton-menubutton-button {
  min-width: 57px;
}

toolbarbutton:not([disabled="true"]):hover,
menubutton:not([disabled="true"]):hover,
toolbarbutton:not([disabled="true"]):hover:active,
menubutton:not([disabled="true"]):hover:active {
	color: ButtonText !important;
}

/* Set the minimum sidebar width so the help contents aren't squeezed together.*/
#helpsidebar-box { width: 200px; }

#help-back-button, 
#help-forward-button[chromedir="rtl"] { -moz-image-region: rect(0px 24px 24px 0px); }
#help-back-button:hover, 
#help-forward-button[chromedir="rtl"]:hover { -moz-image-region: rect(24px 24px 48px 0px); }
#help-back-button[disabled="true"], 
#help-forward-button[disabled="true"][chromedir="rtl"] { -moz-image-region: rect(48px 24px 72px 0px); }

#help-forward-button, #help-back-button[chromedir="rtl"] { -moz-image-region: rect(0px 48px 24px 24px); }
#help-forward-button:hover, 
#help-back-button[chromedir="rtl"]:hover { -moz-image-region: rect(24px 48px 48px 24px); }
#help-forward-button[disabled="true"], 
#help-back-button[disabled="true"][chromedir="rtl"] { -moz-image-region: rect(48px 48px 72px 24px); }

#help-find-button { -moz-image-region: rect(0px 192px 24px 168px); }
#help-find-button:hover { -moz-image-region: rect(24px 192px 48px 168px); }

#help-toolbar-customization { -moz-image-region: rect(0px 216px 24px 192px); }
#help-toolbar-customization:hover { -moz-image-region: rect(24px 216px 48px 192px); }

#help-home-button { -moz-image-region: rect(0px 120px 24px 96px); }
#help-home-button:hover { -moz-image-region: rect(24px 120px 48px 96px); }

#help-print-button { -moz-image-region: rect(0px 216px 24px 192px); }
#help-print-button:hover { -moz-image-region: rect(24px 216px 48px 192px); }

#help-zoom-large-button { -moz-image-region: rect(0px 24px 24px 0px); }
#help-zoom-large-button:hover { -moz-image-region: rect(24px 24px 48px 0px); }

#help-zoom-small-button { -moz-image-region: rect(0px 48px 24px 24px); }
#help-zoom-small-button:hover { -moz-image-region: rect(24px 48px 48px 24px); }

#help-sidebar-button { -moz-image-region: rect(0px 72px 24px 48px); }
#help-sidebar-button:hover { -moz-image-region: rect(24px 72px 48px 48px); }

/** Toolbar Customization stuff **/
toolbar[mode="icons"] > toolbarbutton > .toolbarbutton-icon,
toolbar[mode="text"] > toolbarbutton > .toolbarbutton-text, 
toolbar[mode="icons"] > toolbarbutton[type="menu-button"] > .toolbarbutton-icon,
toolbar[mode="text"] > toolbarbutton[type="menu-button"] > .toolbarbutton-text {
	display: -moz-box;
}

toolbar[mode="icons"] .toolbarbutton-text,
toolbar[mode="text"] .toolbarbutton-icon {
	display: none;
}

/* Small icons are not supported. */
#smallicons { display: none; }

/*** THROBBER ***/
#help-throbber {
  -moz-appearance: none;
  -moz-user-focus: ignore;
  margin: 0 !important;
  border: none !important;
  padding: 0 !important;
  min-width: 0;
  background-color: transparent;
  list-style-image: url("chrome://browser/skin/Throbber-small.png");
}

#help-throbber[busy="true"] {
  list-style-image: url("chrome://browser/skin/Throbber-small.gif");
  opacity: 0.5;
}
