/*
# -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is OEone Calendar Code, released October 31st, 2001.
#
# The Initial Developer of the Original Code is
# OEone Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s): Garth Smedley <garths@oeone.com>
#                 Mike Potter <mikep@oeone.com>
#                 Karl Guertin <grayrest@grayrest.com>
#                 Colin Phillips <colinp@oeone.com>
#                 ArentJan Banck <ajbanck@planet.nl>
#                 Eric Belhaire <belhaire@ief.u-psud.fr>
#                 Mark Swaffer <swaff@fudo.org>
#                 Christopher Cook <tangent@intraplanar.net>
#                 Kevin Gerich <webmail@kmgerich.com>
#                 Matthew Willis <mattwillis@gmail.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
*/


/*--------------------------------------------------------------------
 *   Top bar, with name, image and buttons
 *-------------------------------------------------------------------*/

#calendar-deck {
    overflow: auto;
}

/*--------------------------------------------------------------------
 *   Top bar, with name, image and buttons
 *-------------------------------------------------------------------*/

#calendar-controls-box {
    min-height: 33px;
    max-height: 33px;
}

/*--------------------------------------------------------------------
 *   Icons-only and text-only mode
 *-------------------------------------------------------------------*/

toolbar[mode="icons"] .toolbarbutton-text {
  display: none;
}

toolbar[mode="text"] .toolbarbutton-icon {
  display: none;
}

toolbar[mode="text"] .toolbarbutton-text {
  padding: 0px !important;
  margin: 0px !important;
}

/*--------------------------------------------------------------------
 *   Primary toolbar buttons
 *-------------------------------------------------------------------*/

.toolbarbutton-1 {
    -moz-box-orient: vertical;
    min-width: 0px;
    list-style-image: url("chrome://calendar/skin/toolbar-large.png");
}

/*--------------------------------------------------------------------
 *   Primary toolbar buttons - small
 *-------------------------------------------------------------------*/

toolbar[iconsize="small"] .toolbarbutton-1 {
    -moz-box-orient: vertical;
    min-width: 0px;
    list-style-image: url("chrome://calendar/skin/toolbar-small.png");
}

/*--------------------------------------------------------------------
 *   Button: "New Event"
 *-------------------------------------------------------------------*/

#calendar-new-event-button {
  -moz-image-region: rect(0px 32px 32px 0px);
}
#calendar-new-event-button:active {
    -moz-image-region: rect(32px 32px 64px 0px);
}
#calendar-new-event-button[disabled="true"] {
    -moz-image-region: rect(64px 32px 96px 0px);
}

toolbar[iconsize="small"] #calendar-new-event-button {
    -moz-image-region: rect(0px 24px 24px 0px);
}
toolbar[iconsize="small"] #calendar-new-event-button:active {
    -moz-image-region: rect(24px 24px 48px 0px);
}
toolbar[iconsize="small"] #calendar-new-event-button[disabled="true"] {
    -moz-image-region: rect(48px 24px 72px 0px);
}

/*--------------------------------------------------------------------
 *   Button: "Edit"
 *-------------------------------------------------------------------*/

#calendar-edit-event-button {
    -moz-image-region: rect(0px 64px 32px 32px);
}
#calendar-edit-event-button:active {
    -moz-image-region: rect(32px 64px 64px 32px);
}
#calendar-edit-event-button[disabled="true"] {
    -moz-image-region: rect(64px 64px 96px 32px);
}

toolbar[iconsize="small"] #calendar-edit-event-button {
    -moz-image-region: rect(0px 48px 24px 24px);
}
toolbar[iconsize="small"] #calendar-edit-event-button:active {
    -moz-image-region: rect(24px 48px 48px 24px);
}
toolbar[iconsize="small"] #calendar-edit-event-button[disabled="true"] {
    -moz-image-region: rect(48px 48px 72px 24px);
}

/*--------------------------------------------------------------------
 *   Button: "Delete"
 *-------------------------------------------------------------------*/

#calendar-delete-event-button {
    -moz-image-region: rect(0px 96px 32px 64px);
}
#calendar-delete-event-button:active {
    -moz-image-region: rect(32px 96px 64px 64px);
}
#calendar-delete-event-button[disabled="true"] {
    -moz-image-region: rect(64px 96px 96px 64px);
}

toolbar[iconsize="small"] #calendar-delete-event-button {
    -moz-image-region: rect(0px 72px 24px 48px);
}
toolbar[iconsize="small"] #calendar-delete-event-button:active {
    -moz-image-region: rect(24px 72px 48px 48px);
}
toolbar[iconsize="small"] #calendar-delete-event-button[disabled="true"] {
    -moz-image-region: rect(48px 72px 72px 48px);
}

/*--------------------------------------------------------------------
 *   Go To Today button
 *-------------------------------------------------------------------*/

#calendar-go-to-today-button {
        -moz-image-region: rect(0px 544px 32px 512px);
}
#calendar-go-to-today-button:active {
        -moz-image-region: rect(32px 544px 64px 512px);
}
#calendar-go-to-today-button[disabled="true"] {
        -moz-image-region: rect(64px 544px 96px 512px);
}

toolbar[iconsize="small"] #calendar-go-to-today-button {
        -moz-image-region: rect(0px 408px 24px 384px);
}
toolbar[iconsize="small"] #calendar-go-to-today-button:active {
        -moz-image-region: rect(24px 408px 48px 384px);
}
toolbar[iconsize="small"] #calendar-go-to-today-button[disabled="true"] {
        -moz-image-region: rect(48px 408px 72px 384px);
}

/*--------------------------------------------------------------------
 *   Button: "Choose Date"
 *-------------------------------------------------------------------*/

#calendar-choose-date-button {
        -moz-image-region: rect(0px 576px 32px 544px);
}
#calendar-choose-date-button:active {
        -moz-image-region: rect(32px 576px 64px 544px);
}
#calendar-choose-date-button[disabled="true"] {
        -moz-image-region: rect(64px 576px 96px 544px);
}

toolbar[iconsize="small"] #calendar-choose-date-button {
        -moz-image-region: rect(0px 432px 24px 408px);
}
toolbar[iconsize="small"] #calendar-choose-date-button:active {
        -moz-image-region: rect(24px 432px 48px 408px);
}
toolbar[iconsize="small"] #calendar-choose-date-button[disabled="true"] {
        -moz-image-region: rect(48px 432px 72px 408px);
}

/*--------------------------------------------------------------------
 *   Button: "Multiweek"
 *-------------------------------------------------------------------*/

#calendar-multiweek-view-button {
    -moz-image-region: rect(0px 192px 32px 160px);
}
#calendar-multiweek-view-button:active {
    -moz-image-region: rect(32px 192px 64px 160px);
}
#calendar-multiweek-view-button[disabled="true"] {
    -moz-image-region: rect(96px 192px 128px 160px);
}


toolbar[iconsize="small"] #calendar-multiweek-view-button {
    -moz-image-region: rect(0px 144px 24px 120px);
}
toolbar[iconsize="small"] #calendar-multiweek-view-button:active {
    -moz-image-region: rect(24px 144px 48px 120px);
}
toolbar[iconsize="small"] #calendar-multiweek-view-button[disabled="true"] {
    -moz-image-region: rect(72px 144px 96px 120px);
}

/*--------------------------------------------------------------------
 *   Button: "Month View"
 *-------------------------------------------------------------------*/

#calendar-month-view-button {
    -moz-image-region: rect(0px 224px 32px 192px);
}
#calendar-month-view-button:active {
    -moz-image-region: rect(32px 224px 64px 192px);
}
#calendar-month-view-button:hover:active,
#calendar-month-view-button[disabled="true"] {
    -moz-image-region: rect(96px 224px 128px 192px);
    color: ButtonText;
}


toolbar[iconsize="small"] #calendar-month-view-button {
    -moz-image-region: rect(0px 168px 24px 144px);
}
toolbar[iconsize="small"] #calendar-month-view-button:active {
    -moz-image-region: rect(24px 168px 48px 144px);
}
toolbar[iconsize="small"] #calendar-month-view-button[disabled="true"] {
    -moz-image-region: rect(72px 168px 96px 144px);
}

/*--------------------------------------------------------------------
 *   Button: "Week View"
 *-------------------------------------------------------------------*/

#calendar-week-view-button {
    -moz-image-region: rect(0px 160px 32px 128px);
}
#calendar-week-view-button:active {
    -moz-image-region: rect(32px 160px 64px 128px);
}
#calendar-week-view-button:hover:active,
#calendar-week-view-button[disabled="true"] {
    -moz-image-region: rect(96px 160px 128px 128px);
}


toolbar[iconsize="small"] #calendar-week-view-button {
    -moz-image-region: rect(0px 120px 24px 96px);
}
toolbar[iconsize="small"] #calendar-week-view-button:active {
    -moz-image-region: rect(24px 120px 48px 96px);
}
toolbar[iconsize="small"] #calendar-week-view-button:hover:active,
toolbar[iconsize="small"] #calendar-week-view-button[disabled="true"] {
    -moz-image-region: rect(72px 120px 96px 96px);
}

/*--------------------------------------------------------------------
 *   Button: "Day View"
 *-------------------------------------------------------------------*/

#calendar-day-view-button {
    -moz-image-region: rect(0px 128px 32px 96px);
}
#calendar-day-view-button:active {
    -moz-image-region: rect(32px 128px 64px 96px);
}
#calendar-day-view-button:hover:active,
#calendar-day-view-button[disabled="true"] {
    -moz-image-region: rect(96px 128px 128px 96px);
}


toolbar[iconsize="small"] #calendar-day-view-button {
    -moz-image-region: rect(0px 96px 24px 72px);
}
toolbar[iconsize="small"] #calendar-day-view-button:active {
    -moz-image-region: rect(24px 96px 48px 72px);
}
toolbar[iconsize="small"] #calendar-day-view-button:hover:active,
toolbar[iconsize="small"] #calendar-day-view-button[disabled="true"] {
    -moz-image-region: rect(72px 96px 96px 72px);
}

/*--------------------------------------------------------------------
 *   Button: "Cut"
 *-------------------------------------------------------------------*/

#calendar-cut-button {
        -moz-image-region: rect(0px 256px 32px 224px);
}
#calendar-cut-button:active {
        -moz-image-region: rect(32px 256px 64px 224px);
}
#calendar-cut-button[disabled="true"] {
        -moz-image-region: rect(64px 256px 96px 224px);
}

toolbar[iconsize="small"] #calendar-cut-button {
        -moz-image-region: rect(0px 192px 24px 168px);
}
toolbar[iconsize="small"] #calendar-cut-button:active {
        -moz-image-region: rect(24px 192px 48px 168px);
}
toolbar[iconsize="small"] #calendar-cut-button[disabled="true"] {
        -moz-image-region: rect(48px 192px 72px 168px);
}

/*--------------------------------------------------------------------
 *   Button: "Copy"
 *-------------------------------------------------------------------*/

#calendar-copy-button {
        -moz-image-region: rect(0px 288px 32px 256px);
}
#calendar-copy-button:active {
        -moz-image-region: rect(32px 288px 64px 256px);
}
#calendar-copy-button[disabled="true"] {
        -moz-image-region: rect(64px 288px 96px 256px);
}

toolbar[iconsize="small"] #calendar-copy-button {
        -moz-image-region: rect(0px 216px 24px 192px);
}
toolbar[iconsize="small"] #calendar-copy-button:active {
        -moz-image-region: rect(24px 216px 48px 192px);
}
toolbar[iconsize="small"] #calendar-copy-button[disabled="true"] {
        -moz-image-region: rect(48px 216px 72px 192px);
}

/*--------------------------------------------------------------------
 *   Button: "Paste"
 *-------------------------------------------------------------------*/

#calendar-paste-button {
        -moz-image-region: rect(0px 320px 32px 288px);
}
#calendar-paste-button:active {
        -moz-image-region: rect(32px 320px 64px 288px);
}
#calendar-paste-button[disabled="true"] {
        -moz-image-region: rect(64px 320px 96px 288px);
}

toolbar[iconsize="small"] #calendar-paste-button {
        -moz-image-region: rect(0px 240px 24px 216px);
}
toolbar[iconsize="small"] #calendar-paste-button:active {
        -moz-image-region: rect(24px 240px 48px 216px);
}
toolbar[iconsize="small"] #calendar-paste-button[disabled="true"] {
        -moz-image-region: rect(48px 240px 72px 216px);
}

/*--------------------------------------------------------------------
 *   Button: "Import"
 *-------------------------------------------------------------------*/

#calendar-import-button {
        -moz-image-region: rect(0px 352px 32px 320px);
}
#calendar-import-button:active {
        -moz-image-region: rect(32px 352px 64px 320px);
}
#calendar-import-button[disabled="true"] {
        -moz-image-region: rect(64px 352px 96px 320px);
}

toolbar[iconsize="small"] #calendar-import-button {
        -moz-image-region: rect(0px 264px 24px 240px);
}
toolbar[iconsize="small"] #calendar-import-button:active {
        -moz-image-region: rect(24px 264px 48px 240px);
}
toolbar[iconsize="small"] #calendar-import-button[disabled="true"] {
        -moz-image-region: rect(48px 264px 72px 240px);
}

/*--------------------------------------------------------------------
 *   Button: "Export"
 *-------------------------------------------------------------------*/

#calendar-export-button {
        -moz-image-region: rect(0px 384px 32px 352px);
}
#calendar-export-button:active {
        -moz-image-region: rect(32px 384px 64px 352px);
}
#calendar-export-button[disabled="true"] {
        -moz-image-region: rect(64px 384px 96px 352px);
}

toolbar[iconsize="small"] #calendar-export-button {
        -moz-image-region: rect(0px 288px 24px 264px);
}
toolbar[iconsize="small"] #calendar-export-button:active {
        -moz-image-region: rect(24px 288px 48px 264px);
}
toolbar[iconsize="small"] #calendar-export-button[disabled="true"] {
        -moz-image-region: rect(48px 288px 72px 264px);
}

/*--------------------------------------------------------------------
 *   Button: "Subscribe"
 *-------------------------------------------------------------------*/

#calendar-subscribe-button {
        -moz-image-region: rect(0px 416px 32px 384px);
}
#calendar-subscribe-button:active {
        -moz-image-region: rect(32px 416px 64px 384px);
}
#calendar-subscribe-button[disabled="true"] {
        -moz-image-region: rect(64px 416px 96px 384px);
}

toolbar[iconsize="small"] #calendar-subscribe-button {
        -moz-image-region: rect(0px 312px 24px 288px);
}
toolbar[iconsize="small"] #calendar-subscribe-button:active {
        -moz-image-region: rect(24px 312px 48px 288px);
}
toolbar[iconsize="small"] #calendar-subscribe-button[disabled="true"] {
        -moz-image-region: rect(48px 312px 72px 288px);
}

/*--------------------------------------------------------------------
 *   Button: "Publish"
 *-------------------------------------------------------------------*/

#calendar-publish-button {
        -moz-image-region: rect(0px 448px 32px 416px);
}
#calendar-publish-button:active {
        -moz-image-region: rect(32px 448px 64px 416px);
}
#calendar-publish-button[disabled="true"] {
        -moz-image-region: rect(64px 448px 96px 416px);
}

toolbar[iconsize="small"] #calendar-publish-button {
        -moz-image-region: rect(0px 336px 24px 312px);
}
toolbar[iconsize="small"] #calendar-publish-button:active {
        -moz-image-region: rect(24px 336px 48px 312px);
}
toolbar[iconsize="small"] #calendar-publish-button[disabled="true"] {
        -moz-image-region: rect(48px 336px 72px 312px);
}

/*--------------------------------------------------------------------
 *   Button: "Local Reload"
 *-------------------------------------------------------------------*/

/*#calendar-local-reload-button {
        -moz-image-region: rect(0px 608px 32px 576px);
}
#calendar-local-reloadbuttondown="true"] {
        -moz-image-region: rect(32px 608px 64px 576px);
}
#calendar-local-reload[disabled="true"] {
        -moz-image-region: rect(64px 608px 96px 576px);
}
#calendar-local-reload:hover:active {
}

toolbar[iconsize="small"] #calendar-local-reload-button {
        -moz-image-region: rect(0px 456px 24px 432px);
}
toolbar[iconsize="small"] #calendar-local-reloadbuttondown="true"] {
        -moz-image-region: rect(24px 456px 48px 432px);
}
toolbar[iconsize="small"] #calendar-local-reload[disabled="true"] {
        -moz-image-region: rect(48px 456px 72px 432px);
}*/

/*--------------------------------------------------------------------
 *   Button: (Remote) "Reload"
 *-------------------------------------------------------------------*/

#calendar-remote-reload-button {
        -moz-image-region: rect(0px 640px 32px 608px);
}
#calendar-remote-reload-button:active {
        -moz-image-region: rect(32px 640px 64px 608px);
}
#calendar-remote-reload-button[disabled="true"] {
        -moz-image-region: rect(64px 640px 96px 608px);
}

toolbar[iconsize="small"] #calendar-remote-reload-button {
        -moz-image-region: rect(0px 480px 24px 456px);
}
toolbar[iconsize="small"] #calendar-remote-reload-button:active {
        -moz-image-region: rect(24px 480px 48px 456px);
}
toolbar[iconsize="small"] #calendar-remote-reload-button[disabled="true"] {
        -moz-image-region: rect(48px 480px 72px 456px);
}

/*--------------------------------------------------------------------
 *   Button: "Print"
 *-------------------------------------------------------------------*/

#calendar-print-button {
        -moz-image-region: rect(0px 480px 32px 448px);
}
#calendar-print-button:active {
        -moz-image-region: rect(32px 480px 64px 448px);
}
#calendar-print-button[disabled="true"] {
        -moz-image-region: rect(64px 480px 96px 448px);
}

toolbar[iconsize="small"] #calendar-print-button {
        -moz-image-region: rect(0px 360px 24px 336px);
}
toolbar[iconsize="small"] #calendar-print-button:active {
        -moz-image-region: rect(24px 360px 48px 336px);
}
toolbar[iconsize="small"] #calendar-print-button[disabled="true"] {
        -moz-image-region: rect(48px 360px 72px 336px);
}

/*--------------------------------------------------------------------
 *   Button: "New Task"
 *-------------------------------------------------------------------*/

#calendar-new-task-button {
        -moz-image-region: rect(0px 512px 32px 480px);
}
#calendar-new-task-button:active {
        -moz-image-region: rect(32px 512px 64px 480px);
}
#calendar-new-task-button[disabled="true"] {
        -moz-image-region: rect(64px 512px 96px 480px);
}

toolbar[iconsize="small"] #calendar-new-task-button {
        -moz-image-region: rect(0px 384px 24px 360px);
}
toolbar[iconsize="small"] #calendar-new-task-button:active {
        -moz-image-region: rect(24px 384px 48px 360px);
}
toolbar[iconsize="small"] #calendar-new-task-button[disabled="true"] {
        -moz-image-region: rect(48px 384px 72px 360px);
}

/*--------------------------------------------------------------------
 *   Throbber button
 *-------------------------------------------------------------------*/

/* "Chasing pipes" Mac-style throbber */
#navigator-throbber {
  -moz-appearance: none;
  -moz-user-focus: ignore;
  -moz-box-pack: center;
  margin: 0px 1px 0px 1px !important;
  border: none !important;
  padding: 0px !important;
  min-width: 0;
  background-color: transparent;
  list-style-image: url("chrome://global/skin/icons/notloading_16.png");
  padding: 8px !important;
}

#navigator-throbber[busy="true"] {
  list-style-image: url("chrome://global/skin/icons/loading_16.gif");
}

toolbar[iconsize="small"] #navigator-throbber,
toolbar[mode="text"] #navigator-throbber {
  list-style-image: url("chrome://global/skin/icons/notloading_16.png");
  padding: 4px !important;
}
toolbar[iconsize="small"] #navigator-throbber[busy="true"],
toolbar[mode="text"] #navigator-throbber[busy="true"] {
  list-style-image: url("chrome://global/skin/icons/loading_16.gif");
}

toolbarbutton.chevron {
  list-style-image: url("chrome://global/skin/icons/chevron.png") !important;
}

toolbarbutton.chevron > .toolbarbutton-menu-dropmarker {
  display: none;
}

/* UNIFINDER */

.calendar-unifinder-event-indent
{
      padding-left        : 30px;
}

.unifinder-treecell-box-class
{
   padding              : 2px;
   padding-left         : 4px;
   font-family          : Arial, Helvetica;
   -moz-box-align       : start;
}


/* TO DO ITEMS */

treechildren::-moz-tree-cell-text(inprogress)
{
   color                 : green;
}

treechildren::-moz-tree-row(inprogress,selected)
{
   background-color                 : green;
}

treechildren::-moz-tree-cell-text(overdue)
{
   color                  : red;
}

treechildren::-moz-tree-row(overdue,selected)
{
   background-color                  : red;
}

treechildren::-moz-tree-row
{
   border-bottom             : 1px dotted #AAA;
}

treechildren::-moz-tree-cell-text(duetoday)
{
   color                  : blue;
}

treechildren::-moz-tree-row(duetoday,selected)
{
   background-color                  : blue;
}

treechildren::-moz-tree-cell-text(future)
{
   color                  : gray;
}

treechildren::-moz-tree-row(future,selected)
{
   background-color                  : gray;
}

treechildren::-moz-tree-cell-text(completed)
{
   text-decoration        : line-through;
   color                  : black;
}


/* TO DO LIST IMAGES */

treechildren::-moz-tree-image(unifinder-todo-tree-col-completed, completed),
#unifinder-todo-tree-col-completed {
   list-style-image      : url("chrome://calendar/skin/unifinder/checkbox_checked.png");
}

treechildren::-moz-tree-image(unifinder-todo-tree-col-completed, duetoday),
treechildren::-moz-tree-image(unifinder-todo-tree-col-completed, overdue),
treechildren::-moz-tree-image(unifinder-todo-tree-col-completed, future),
treechildren::-moz-tree-image(unifinder-todo-tree-col-completed, inprogress) {
   list-style-image      : url("chrome://calendar/skin/unifinder/checkbox_unchecked.png");
}

#unifinder-todo-tree-col-priority {
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_header.png" )
}

treechildren::-moz-tree-image(unifinder-todo-tree-col-priority, mediumpriority) {
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_header.png" )
}

treechildren::-moz-tree-image(unifinder-todo-tree-col-priority, highpriority) {
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_high.png" )
}

treechildren::-moz-tree-image(unifinder-todo-tree-col-priority, lowpriority) {
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_low.png" )
}

treechildren::-moz-tree-cell-text(selected) {
   color                  : HighlightText;
}

.calendar-list-item-class[synching="true"]
{
   list-style-image     : url( "chrome://calendar/skin/synch_animated.gif" );
}

listitem[selected="true"] > .calendar-list-item-class
{
  background-color     : White;
  color                : Black;
}

.todo-due-image-class
{
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_header.png" )
}

.todo-due-image-class[highpriority="true"]
{
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_high.png" )
}

.todo-due-image-class[lowpriority="true"]
{
   list-style-image      : url( "chrome://calendar/skin/unifinder/priority_low.png" )
}
/*--------------------------------------------------------------------
 * Unifinder
 *-------------------------------------------------------------------*/
/* workaround to avoid Window Flick */
#unifinder-search-results-listbox
{
   min-height : 40px ;
}

/*--------------------------------------------------------------------
 * tooltips
 *-------------------------------------------------------------------*/
label.tooltipHeaderLabel
{
  font-weight : bold;
  text-align  : right;
}

description.tooltipHeaderDescription
{
  font-weight : normal;
  text-align  : left;
  white-space : normal;
}

description.tooltipBody
{
  font-weight : normal;
  white-space : normal;
}

/*--------------------------------------------------------------------
 * print preview
 *-------------------------------------------------------------------*/
toolbar[printpreview="true"] {
  -moz-binding: url("chrome://global/content/printPreviewBindings.xml#printpreviewtoolbar");
}

@import url("chrome://calendar/skin/calendar-views.css");

